-- *****************************************************
-- Copyright (c) 2001-2007 iAnywhere Solutions, Inc.
-- Portions copyright (c) 1988-2007 Sybase, Inc.
-- All rights reserved. All unpublished rights reserved.
-- *****************************************************

--
-- Create the MobiLink Server system tables and stored procedures in
-- a Microsoft SQL Server consolidated database.
--

set ANSI_PADDING OFF
go
create table ml_user (
    user_id		    integer	    not null,
    name		    varchar( 128 )  not null unique,
    hashed_password	    binary( 32 )    null,
    primary key( user_id ) )
go

create table ml_database (
    rid			    integer	    not null,
    remote_id		    varchar( 128 )  not null unique,
    description		    varchar( 128 )  null,
    primary key( rid ) )
go

create table ml_subscription (
    rid			    integer	    not null,
    subscription_id	    varchar( 128 )  default '<unknown>' not null,
    user_id		    integer	    not null,
    progress		    numeric( 20 )   default 0 not null,
    publication_name	    varchar( 128 )  default '<unknown>' not null,
    last_upload_time	    datetime	    default CONVERT( DATETIME, '01/01/1900', 101 ) not null,
    last_download_time	    datetime	    default CONVERT( DATETIME, '01/01/1900', 101 ) not null,
    primary key( rid, subscription_id ),
    foreign key( rid ) references ml_database ( rid ),
    foreign key( user_id ) references ml_user ( user_id ) )
go

create table ml_table (
    table_id		integer			not null,
    name		varchar( 128 )     	not null unique,
    primary key( table_id ) )
go
    
create table ml_script (
    script_id		integer			not null,
    script		text			not null,
    script_language	varchar( 128 )		default 'sql' not null
    ,
    checksum		varchar( 64 )		null
    ,
    primary key( script_id ) )
go

create table ml_script_version (
    version_id		integer			not null, 
    name		varchar( 128 )		not null unique,
    description		text			null,
    primary key( version_id ) )
go
       
create table ml_connection_script (
    version_id		integer			not null,
    event		varchar( 128 )     	not null,
    script_id		integer			not null,
    foreign key( version_id ) references ml_script_version ( version_id ),
    foreign key( script_id ) references ml_script ( script_id ),
    primary key( version_id, event ) )
go
    
create table ml_table_script (
    version_id		integer			not null,
    table_id		integer			not null,
    event		varchar( 128 )		not null,
    script_id		integer			not null,
    foreign key( version_id ) references ml_script_version ( version_id ),
    foreign key( table_id ) references ml_table ( table_id ),
    foreign key( script_id ) references ml_script ( script_id ),
    primary key( version_id, table_id, event ) )
go

create table ml_property (
    component_name		varchar( 128 )	not null,
    property_set_name		varchar( 128 )	not null,
    property_name		varchar( 128 )	not null,
    property_value		text		not null,
    primary key( component_name, property_set_name, property_name ) )
go
    
create table ml_scripts_modified (
    last_modified	datetime		primary key not null )
go

delete from ml_scripts_modified
go

insert into ml_scripts_modified ( last_modified ) values ( GETDATE() )
go

create trigger ml_script_trigger 
    on ml_script
    for delete, insert, update
as
begin
    update ml_scripts_modified set last_modified = GETDATE();
end
go
    
create trigger ml_table_script_trigger 
    on ml_table_script
    for delete, insert, update
as
begin
    update ml_scripts_modified set last_modified = GETDATE();
end
go
    
create trigger ml_connection_script_trigger 
    on ml_connection_script
    for delete, insert, update
as
begin
    update ml_scripts_modified set last_modified = GETDATE();
end
go

create table ml_column (
    version_id	integer		not null,
    table_id	integer		not null,
    idx		integer		not null,
    name	varchar( 128 )	not null,
    type	varchar( 128 )	null,
    primary key( idx, version_id, table_id ),
    unique( version_id, table_id, name ),
    foreign key( version_id ) references ml_script_version ( version_id ),
    foreign key( table_id ) references ml_table ( table_id ) )
go

create view ml_connection_scripts as
select ml_script_version.name version,
       ml_connection_script.event,
       ml_script.script_language,
       ml_script.script
from ml_connection_script,
     ml_script_version,
     ml_script
where ml_connection_script.version_id = ml_script_version.version_id
and ml_connection_script.script_id = ml_script.script_id
go

create view ml_table_scripts as
select ml_script_version.name version,
       ml_table_script.event,
       ml_table.name table_name,
       ml_script.script_language,
       ml_script.script
from ml_table_script,
     ml_script_version,
     ml_script,
     ml_table
where ml_table_script.version_id = ml_script_version.version_id
and ml_table_script.script_id = ml_script.script_id
and ml_table_script.table_id = ml_table.table_id
go

create procedure ml_add_user
    @user	varchar( 128 ),
    @password	binary( 32 )
as
begin
    declare	@user_id	integer

    set nocount on
    
    select @user_id = user_id from ml_user where name = @user
    if @user_id is null begin
	select @user_id = max( user_id )+1 from ml_user
	if @user_id is null begin
	    select @user_id = 1
	end
	insert into ml_user ( user_id, name, hashed_password )
	    values ( @user_id, @user, @password )
    end
    else begin
    	update ml_user set hashed_password = @password where name = @user
    end
end
go

create procedure ml_delete_user
    @user	varchar( 128 )
as
begin
    declare	@uid	integer
    declare	@rid	integer
    
    set nocount on
    
    select @uid = user_id from ml_user where name = @user
    if @uid is not null begin
	declare	crsr cursor for select rid from ml_subscription
				    where user_id = @uid for read only
	open crsr
	fetch crsr into @rid
	while( @@FETCH_STATUS = 0 )
	begin
	    delete from ml_subscription where user_id = @uid and rid = @rid
	    if not exists (select * from ml_subscription where rid = @rid)
		delete from ml_database where rid = @rid
	    fetch crsr into @rid
	end
	close crsr
	deallocate crsr
	delete from ml_user where user_id = @uid
    end
end
go

create procedure ml_delete_sync_state
    @user		varchar( 128 ),
    @remote_id		varchar( 128 )
as
begin
    declare	@uid	integer
    declare	@rid	integer
    
    set nocount on
    
    select @uid = user_id from ml_user where name = @user
    select @rid = rid from ml_database where remote_id = @remote_id
    
    if @user is not null and @remote_id is not null begin
	delete from ml_subscription where user_id = @uid and rid = @rid
	if not exists (select * from ml_subscription where rid = @rid)
	    delete from ml_database where rid = @rid
    end
    else if @user is not null begin
	declare	crsr cursor for select rid from ml_subscription
				    where user_id = @uid for read only
	open crsr
	fetch crsr into @rid
	while( @@FETCH_STATUS = 0 )
	begin
	    delete from ml_subscription where user_id = @uid and rid = @rid
	    if not exists (select * from ml_subscription where rid = @rid)
		delete from ml_database where rid = @rid
	    fetch crsr into @rid
	end
	close crsr
	deallocate crsr
    end
    else if @remote_id is not null begin
	delete from ml_subscription where rid = @rid
	delete from ml_database where rid = @rid
    end
end
go

create procedure ml_reset_sync_state
    @user		varchar( 128 ),
    @remote_id		varchar( 128 )
as
begin
    declare	@uid	integer
    declare	@rid	integer

    set nocount on
    
    select @uid = user_id from ml_user where name = @user
    select @rid = rid from ml_database where remote_id = @remote_id
    
    if @user is not null and @remote_id is not null begin
	update ml_subscription
	    set progress = 0,
		last_upload_time   = CONVERT( DATETIME, '01/01/1900', 101 ),
		last_download_time = CONVERT( DATETIME, '01/01/1900', 101 )
	    where user_id = @uid and rid = @rid
    end
    else if @user is not null begin 
	update ml_subscription
	    set progress = 0,
		last_upload_time   = CONVERT( DATETIME, '01/01/1900', 101 ),
		last_download_time = CONVERT( DATETIME, '01/01/1900', 101 )
	    where user_id = @uid
    end
    else if @remote_id is not null begin	
	update ml_subscription
	    set progress = 0,
		last_upload_time   = CONVERT( DATETIME, '01/01/1900', 101 ),
		last_download_time = CONVERT( DATETIME, '01/01/1900', 101 )
	    where rid = @rid
    end
end
go

create procedure ml_delete_sync_state_before @ts datetime
as
begin
    declare	@rid	integer
    
    set nocount on
    
    if @ts is not null begin
	declare	crsr cursor for select rid from ml_subscription
				    where last_upload_time < @ts and
					  last_download_time < @ts
				    for read only
	open crsr
	fetch crsr into @rid
	while( @@FETCH_STATUS = 0 )
	begin
	    delete from ml_subscription where rid = @rid and
					      last_upload_time < @ts and
					      last_download_time < @ts
	    if not exists (select * from ml_subscription where rid = @rid)
		delete from ml_database where rid = @rid
	    fetch crsr into @rid
	end
	close crsr
	deallocate crsr
    end
end
go


create procedure ml_add_lang_table_script_chk
    @version		varchar( 128 ),
    @table		varchar( 128 ),
    @event		varchar( 128 ),
    @script_language	varchar( 128 ),
    @script		text,
    @checksum		varchar( 64 )
as
begin
    declare @version_id 	integer;
    declare @table_id 		integer;
    declare @script_id 		integer;
    declare @upd_script_id	integer;
    
    set nocount on
    
    select @version_id = version_id from ml_script_version 
        where name = @version;
    select @table_id = table_id from ml_table where name = @table;
    if @script is not null begin
	if @version_id is null begin
	    -- Insert to the ml_script_version
	    select @version_id = max( version_id )+1 from ml_script_version;
	    if @version_id is null begin
		-- No rows are currently in ml_script_version
		set @version_id = 1;
	    end;
	    insert into ml_script_version ( version_id, name )
		    values ( @version_id, @version );
	end;
	
	if @table_id is null begin
	    -- Insert to the ml_table
	    select @table_id = max( table_id )+1 from ml_table;
	    if @table_id is null begin
		-- No rows are currently in ml_table
		set @table_id = 1;			
	    end;
	    insert into ml_table ( table_id, name ) 
	        values ( @table_id, @table );
	end;
	
	select @script_id = max( script_id )+1 from ml_script;
	if @script_id is null begin
	    -- No rows are currently in ml_script
	    set @script_id = 1;
	end;
	insert into ml_script ( script_id, script_language, script, checksum ) 
    		values ( @script_id, @script_language, @script, @checksum );
	select @upd_script_id = script_id from ml_table_script where version_id = @version_id and table_id = @table_id and event = @event
	if @upd_script_id is null begin
	    insert into ml_table_script ( version_id, table_id, event, script_id ) 
		values ( @version_id, @table_id, @event, @script_id );
	    end;
	else begin
	    update ml_table_script set script_id = @script_id where version_id = @version_id and table_id = @table_id and event = @event
	end;
	end;
    else begin
	if @version_id is not null and @table_id is not null begin
	    delete from ml_table_script where version_id = @version_id 
	        and table_id = @table_id and event = @event;
	end;
    end;
end
go

create procedure ml_add_lang_table_script
    @version		varchar( 128 ),
    @table		varchar( 128 ),
    @event		varchar( 128 ),
    @script_language	varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_table_script_chk @version, @table, @event, @script_language, @script, null
end
go

create procedure ml_add_table_script
    @version		varchar( 128 ),
    @table		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_table_script @version, @table, @event, 'sql', @script
end
go

create procedure ml_add_java_table_script
    @version		varchar( 128 ),
    @table		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_table_script @version, @table, @event, 'java', @script
end
go

create procedure ml_add_dnet_table_script
    @version		varchar( 128 ),
    @table		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_table_script @version, @table, @event, 'dnet', @script
end
go

create procedure ml_add_lang_conn_script_chk
    @version		varchar( 128 ),
    @event		varchar( 128 ),
    @script_language	varchar( 128 ),
    @script		text,
    @checksum		varchar( 64 )
as
begin
    declare @version_id 	integer;
    declare @script_id 		integer;
    declare @upd_script_id	integer;
    
    set nocount on

    select @version_id = version_id from ml_script_version 
        where name = @version;
    if @script is not null begin
	if @version_id is null begin
	    -- Insert to the ml_script_version
	    select @version_id = max( version_id )+1 from ml_script_version;
	    if @version_id is null begin
		-- No rows are currently in ml_script_version
		set @version_id = 1;
	    end;
	    insert into ml_script_version ( version_id, name ) 
		    values ( @version_id, @version );
	end;
    
	select @script_id = max( script_id )+1 from ml_script;
	if @script_id is null begin
	    -- No rows are currently in ml_script
	    set @script_id = 1;
	end;
	insert into ml_script ( script_id, script_language, script, checksum ) 
    		values ( @script_id, @script_language, @script, @checksum );
	select @upd_script_id = script_id from ml_connection_script 
	    where version_id = @version_id and event = @event
	if @upd_script_id is null begin
	    insert into ml_connection_script ( version_id, event, script_id ) 
    		values ( @version_id, @event, @script_id );
	end;
	else begin
	    update ml_connection_script set script_id = @script_id 
	        where version_id = @version_id and event = @event
	end;
    end;
    else begin
	if @version_id is not null begin
	    delete from ml_connection_script 
	        where version_id = @version_id and event = @event;
	end;
    end;
end
go

create procedure ml_add_lang_connection_script
    @version		varchar( 128 ),
    @event		varchar( 128 ),
    @script_language	varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_conn_script_chk @version, @event, @script_language, @script, null
end
go

create procedure ml_add_connection_script
    @version		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_connection_script @version, @event, 'sql', @script
end
go

create procedure ml_add_java_connection_script
    @version		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_connection_script @version, @event, 'java', @script
end
go

create procedure ml_add_dnet_connection_script
    @version		varchar( 128 ),
    @event		varchar( 128 ),
    @script		text
as
begin
    exec ml_add_lang_connection_script @version, @event, 'dnet', @script
end
go

create procedure ml_add_property
    @comp_name	    varchar( 128 ),
    @prop_set_name  varchar( 128 ),
    @prop_name	    varchar( 128 ),
    @prop_value	    text
as
begin
    declare 	@p_name 	varchar( 128 );

    if @prop_value is null begin
	delete from ml_property where component_name  = @comp_name
				and property_set_name = @prop_set_name
				and property_name     = @prop_name;
        end;
    else begin
	select @p_name = property_name from ml_property 
				    where component_name  = @comp_name
				    and property_set_name = @prop_set_name
				    and property_name	  = @prop_name;
	if @p_name is null begin
	    insert into ml_property
		(component_name,property_set_name,property_name,property_value)
		values (@comp_name, @prop_set_name, @prop_name, @prop_value );
	    end;
	else begin
	    update ml_property set property_value = @prop_value
				    where component_name  = @comp_name
				    and property_set_name = @prop_set_name
				    and property_name	  = @prop_name;
	end;
    end;
end
go

create procedure ml_add_column 
    @version	varchar( 128 ),
    @table	varchar( 128 ),
    @column	varchar( 128 ),
    @type	varchar( 128 )
as
begin
    declare 	@version_id	integer
    declare 	@table_id	integer
    declare 	@idx		integer
    
    select @version_id = version_id from ml_script_version 
        where name = @version;
    select @table_id = table_id from ml_table where name = @table;
    if @column is not null begin
    	if @version_id is null begin
	    -- Insert to the ml_script_version table
	    select @version_id = max( version_id )+1 from ml_script_version;
	    if @version_id is null begin
	    	-- No rows are currently in ml_script_version
	    	select @version_id = 1;
	    end;
	    insert into ml_script_version ( version_id, name ) 
		    values ( @version_id, @version );
    	end;

    	if @table_id is null begin
	    -- Insert to the ml_table table
	    select @table_id = max( table_id )+1 from ml_table;
	    if @table_id is null begin
	    	-- No rows are currently in ml_table
	    	select @table_id = 1;
	    end;		
	    insert into ml_table ( table_id, name ) 
	        values ( @table_id, @table );
    	end;
    
	select @idx = max( idx )+1 from ml_column
	    where version_id = @version_id and table_id = @table_id;
	if @idx is null begin
	    select @idx = 1;
	end
	insert into ml_column ( version_id, table_id, idx, name, type ) 
	    values ( @version_id, @table_id, @idx, @column, @type );
    end;
    else begin
	if @version_id is not null and @table_id is not null begin
	    delete from ml_column 
	        where version_id = @version_id and table_id = @table_id;
	end;
    end;
end
go

create table ml_device (
    device_name		varchar( 255 )	not null,
    listener_version	varchar( 128 )	not null,
    listener_protocol	integer		not null,
    info		varchar( 255 )	not null,
    ignore_tracking	varchar( 1 )	not null,
    source		varchar( 255 )	not null,
    primary key( device_name )
)
go

create table ml_device_address (
    device_name		varchar( 255 )	not null,
    medium		varchar( 255 )	not null,
    address		varchar( 255 )	not null,
    active		varchar( 1 )	not null,
    last_modified	datetime	not null default GETDATE(),
    ignore_tracking	varchar( 1 )	not null,
    source		varchar( 255 )	not null,
    primary key( device_name, medium ),
    foreign key(device_name) references ml_device ( device_name )
)
go

create table ml_listening (
    name		varchar( 128 )	not null,
    device_name		varchar( 255 )	not null,
    listening		varchar( 1 )	not null,
    ignore_tracking	varchar( 1 )	not null,
    source		varchar( 255 )	not null,
    primary key( name ),
    foreign key( device_name ) references ml_device ( device_name )
)
go

create table ml_sis_sync_state (
    remote_id		varchar( 128 ) not null,
    subscription_id	varchar( 128 ) not null,
    publication_name	varchar( 128 ) not null,
    user_name		varchar( 128 ) not null,
    last_upload		datetime not null,
    last_download	datetime not null,
    primary key( remote_id, subscription_id )
)
go

create procedure ml_set_device
    @device		varchar( 255 ),
    @listener_version	varchar( 128 ),
    @listener_protocol	integer,
    @info		varchar( 255 ),
    @ignore_tracking	varchar( 1 ),
    @source		varchar( 255 )
as
begin
    declare @d		varchar( 255 );

    select @d = device_name from ml_device where device_name = @device;
    if @d is null begin
	insert into ml_device( device_name, listener_version, listener_protocol, info, ignore_tracking, source )
		values( @device, @listener_version, @listener_protocol, @info, @ignore_tracking, @source );
        end;
    else begin
        if @source = 'tracking' begin
	    update ml_device 
		set listener_version = @listener_version,
		    listener_protocol = @listener_protocol,
		    info = @info,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where device_name = @device and ignore_tracking = 'n';
	    end;
	else begin
	    update ml_device 
		set listener_version = @listener_version,
		    listener_protocol = @listener_protocol,
		    info = @info,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where device_name = @device;
	end;
    end;
end
go

create procedure ml_set_device_address
    @device		varchar( 255 ),
    @medium		varchar( 255 ),
    @address		varchar( 255 ),
    @active		varchar( 1 ),
    @ignore_tracking	varchar( 1 ),
    @source		varchar( 255 )
as
begin
    declare @d		varchar( 255 );

    select @d = device_name from ml_device_address
		where device_name = @device and medium = @medium;
    if @d is null begin
	insert into ml_device_address( device_name, medium, address, active, ignore_tracking, source )
		values( @device, @medium, @address, @active, @ignore_tracking, @source );
        end
    else begin
	if @source = 'tracking' begin
	    update ml_device_address 
		set address = @address,
		    active = @active,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where device_name = @device and medium = @medium and ignore_tracking = 'n';
	    end;
	else begin
	    update ml_device_address 
		set address = @address,
		    active = @active,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where device_name = @device and medium = @medium;
	end;
    end;
end
go

create procedure ml_upload_update_device_address
    @address		varchar( 255 ),
    @active		varchar( 1 ),
    @device		varchar( 255 ),
    @medium		varchar( 255 )
as
begin
    exec ml_set_device_address @device, @medium, @address, @active, 'n', 'tracking';
end
go

create procedure ml_set_listening
    @name		varchar( 128 ),
    @device		varchar( 255 ),
    @listening		varchar( 1 ),
    @ignore_tracking	varchar( 1 ),
    @source		varchar( 255 )
as
begin
    declare @d varchar( 255 );

    select @d = device_name from ml_listening where name = @name;
    if @d is null begin
	insert into ml_listening( name, device_name, listening, ignore_tracking, source )
	    values( @name, @device, @listening, @ignore_tracking, @source );
        end;
    else begin
        if @source = 'tracking' begin
	    update ml_listening
		set device_name = @device,
		    listening = @listening,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where name = @name and ignore_tracking = 'n';
	    end;
	else begin
	    update ml_listening
		set device_name = @device,
		    listening = @listening,
		    ignore_tracking = @ignore_tracking,
		    source = @source
		where name = @name;
	end;
    end;
end
go

create procedure ml_set_sis_sync_state
    @remote_id		varchar( 128 ),
    @subscription_id	varchar( 128 ),
    @publication_name	varchar( 128 ),
    @user_name		varchar( 128 ),
    @last_upload	datetime,
    @last_download	datetime
as
begin
    declare @r varchar( 128 );
    
    select @r = remote_id from ml_sis_sync_state
	    where remote_id = @remote_id
	    and subscription_id = @subscription_id;
    if @r is null begin
	insert into ml_sis_sync_state(remote_id, subscription_id, publication_name, user_name, last_upload, last_download )
	    values( @remote_id, @subscription_id, @publication_name, @user_name, @last_upload, @last_download );
	end;
    else begin
	update ml_sis_sync_state
	    set publication_name = @publication_name,
		user_name = @user_name,
		last_upload = @last_upload,
		last_download = @last_download
	    where remote_id = @remote_id
		and subscription_id = @subscription_id;
    end;
end
go

create procedure ml_upload_update_listening
    @device		varchar( 255 ),
    @listening		varchar( 1 ),
    @name		varchar( 128 )
as
begin
    exec ml_set_listening @name, @device, @listening, 'n', 'tracking';
end
go

create procedure ml_delete_device_address
    @device		varchar( 255 ),
    @medium		varchar( 255 )
as
begin
    delete from ml_device_address where device_name = @device and medium = @medium;
end
go

create procedure ml_delete_listening
    @name		varchar( 128 )
as
begin
    delete from ml_listening where name = @name;
end
go

create procedure ml_delete_device 
    @device		varchar( 255 )
as
begin
    delete from ml_device_address where device_name = @device;
    delete from ml_listening where device_name = @device;
    delete from ml_device where device_name = @device;
end
go


exec ml_add_property 'SIS', 'DeviceTracker(Default-DeviceTracker)', 'enable', 'yes'
exec ml_add_property 'SIS', 'DeviceTracker(Default-DeviceTracker)', 'smtp_gateway', 'Default-SMTP'
exec ml_add_property 'SIS', 'DeviceTracker(Default-DeviceTracker)', 'udp_gateway', 'Default-UDP'
exec ml_add_property 'SIS', 'DeviceTracker(Default-DeviceTracker)', 'sync_gateway', 'Default-SYNC'
exec ml_add_property 'SIS', 'SMTP(Default-SMTP)', 'enable', 'yes'
exec ml_add_property 'SIS', 'UDP(Default-UDP)', 'enable', 'yes'
exec ml_add_property 'SIS', 'SYNC(Default-SYNC)', 'enable', 'yes'
exec ml_add_property 'SIS', 'Carrier(Rogers)', 'enable', 'yes'
exec ml_add_property 'SIS', 'Carrier(Rogers)', 'network_provider_id', 'ROGERS'
exec ml_add_property 'SIS', 'Carrier(Rogers)', 'sms_email_user_prefix', '1'
exec ml_add_property 'SIS', 'Carrier(Rogers)', 'sms_email_domain', 'pcs.rogers.com'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility)', 'enable', 'yes'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility)', 'network_provider_id', 'BELL'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility)', 'sms_email_domain', 'txt.bellmobility.ca'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility 1x)', 'enable', 'yes'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility 1x)', 'network_provider_id', 'CDMA1x:16420:65535'
exec ml_add_property 'SIS', 'Carrier(Bell Mobility 1x)', 'sms_email_domain', 'txt.bellmobility.ca'
go



create table ml_qa_repository (
	msgid		varchar(128)	not null,
	props		image	null,
	content	image	null,
	primary key(msgid) )

go


--
-- Out-of-the-box MobiLink QAnywhere Notifier settings
--
exec ml_add_property 'SIS', 'Notifier(QAnyNotifier_client)', 'gui', 'off'
go
exec ml_add_property 'SIS', 'Notifier(QAnyNotifier_client)', 'enable', 'no'
go
exec ml_add_property 'SIS', 'Notifier(QAnyNotifier_client)', 'poll_every', '5'
go
exec ml_add_property 'SIS', 'Notifier(QAnyNotifier_client)', 'request_cursor', 'select u.user_id, ''Default-DeviceTracker'', ''qa'', u.name, u.name, ''5M'', ''3H'' from ml_qa_notifications u where exists( select * from ml_listening l where l.name = u.name and l.listening = ''y'')'
go
exec ml_add_property 'SIS', 'Notifier(QAnyNotifier_client)', 'request_delete', 'delete from ml_qa_notifications where user_id = ?'
go

create table ml_qa_notifications (
    user_id		integer		not null,
    name		varchar(128)	not null,
    primary key(name) )
go



create table ml_qa_delivery (
	msgid		varchar(128)	not null,
	seqno		numeric(20)		not null,
	address		varchar(255)	not null,
	clientaddress	varchar(128)	not null,
	client		varchar(128)	not null,
	originator		varchar(128)	null,
	priority	integer 	not null,
	expires	datetime	null,
	kind		integer 	not null,
	contentsize	numeric(20)		not null,
	status		integer	 	not null,
	statustime	datetime	default getdate() not null,
	syncstatus	integer		not null,
	receiverid	varchar(128)	null,
	
	constraint ml_qa_d_r_c foreign key (msgid) references ml_qa_repository(msgid)
	    on delete cascade,
	
	primary key( msgid, address ) )

go

create table ml_qa_status_history (
	msgid		    varchar(128)	not null,
	address		    varchar(255)	not null,
	status		    integer	not null,
	statustime	    datetime	not null,
	servertime    datetime	default getdate() not null,
	details	    varchar( 1000 ) null,
	syncstatus	    integer	not null
	
	,
	constraint ml_qa_s_h foreign key (msgid) references ml_qa_repository(msgid)
	    on delete cascade
	
	)

go

    create procedure ml_qa_add_message
	@seqno	    numeric(20),
	@msgid	    varchar(128),
	@orig	    varchar(128),
	@priority   integer,
	@expires    datetime,
	@props	    image,
	@kind	    integer,
	@content    image,
	@contentsize integer
    as
    begin
	insert into ml_qa_repository( msgid, props, content )
	    values ( @msgid, @props, @content )
	insert into ml_qa_delivery( msgid, seqno, address, clientaddress,
		client, originator, priority, expires, kind, contentsize,
		status, syncstatus )
	    values ( @msgid, @seqno, 'M', '', '', @orig, @priority,
		@expires, @kind, @contentsize, 0, 0 )
    end
go

    create procedure ml_qa_add_delivery
	@msgid	    varchar(128),
	@addr	    varchar(255),
	@client	    varchar(128),
	@status	    integer,
	@statustime datetime,
	@details    varchar(1000)
    as
    begin
	if @statustime is null
	    select @statustime = getdate()
	update ml_qa_delivery set client = @client, status = @status, 
		statustime = @statustime, syncstatus = 0, address = @addr,
		clientaddress = substring( @addr, 2,
		charindex( '\', @addr )-2)   -- Fix syntax highlighting '
	    where msgid = @msgid and (address = @addr or address = 'M')
    end
go

    create trigger ml_qa_delivery_trigger on ml_qa_delivery for insert, update
    as
    begin

	insert into ml_qa_status_history ( msgid, address, status,
		statustime, syncstatus )
	    select msgid, address, status, statustime, syncstatus
		from inserted i
		where i.address <> 'M'
    end
go



    create table ml_qa_global_props (
	client		varchar(128)		default ' ' not null,
	name		varchar(128)		not null,
	modifiers	integer			not null,
	value		text		null,
	last_modified	datetime
	
	null,
	primary key( client, name ) )

go

    create trigger ml_qa_global_props_trigger
	on ml_qa_global_props for insert, update
    as
    begin
	update ml_qa_global_props set last_modified = getdate()
	    from inserted i
	    where ml_qa_global_props.client = i.client and ml_qa_global_props.name = i.name;
    end
go

create table ml_qa_repository_props (
	msgid		varchar(128)	not null,
	name		varchar(128)	not null,
	value		text null,
	constraint ml_qa_r_p foreign key (msgid) references ml_qa_repository(msgid)
	    on delete cascade,
	primary key( msgid, name ) )
go

    create procedure ml_qa_upsert_global_prop
	@client		varchar(128),
	@name		varchar(128),
	@modifiers	integer,	
	@value		text
    as
    begin
	SET NOCOUNT ON
	if exists( select 1 from ml_qa_global_props gp
		where gp.client = @client and gp.name = @name )
	    update ml_qa_global_props
		set modifiers = @modifiers, value = @value
		where client = @client and name = @name
	else
	    insert into ml_qa_global_props (client,name,modifiers,value)
		values( @client, @name, @modifiers, @value );
	
    end
go


create view ml_qa_messages as
select md.seqno, mr.msgid, md.address, md.client, md.originator, md.status,
    md.statustime, md.expires, md.priority, mr.props, md.kind,
    mr.content, md.contentsize, md.syncstatus, md.receiverid,
    md.clientaddress
    from ml_qa_repository mr, ml_qa_delivery md
    where mr.msgid = md.msgid
go


create index ml_qa_delivery_cli on ml_qa_delivery ( client )
go

create index ml_qa_delivery_ord on ml_qa_delivery ( clientaddress,
		status, priority desc, seqno )
go

create index ml_qa_delivery_dld on ml_qa_delivery ( client, syncstatus,
    status )
go

create index ml_qa_deliver_orig on ml_qa_delivery ( originator )
go



    create procedure ml_qa_handle_error
	@action		integer out,
	@error_code		integer,
	@error_message	varchar( 255 ),
	@user_name		varchar( 128 ),
	@table_name		varchar( 128 ),
	@script_id		varchar( 128 ) = null
    AS
    BEGIN
	SET NOCOUNT ON
	
	-- 2601 is Attempt to insert duplicate row 
	-- 2627 is Attempt to insert duplicate row 
	-- 0547 is INSERT statement conflicted with COLUMN FOREIGN KEY constraint
	-- 3621 indicates an aborted command, and goes hand-in-hand with other
	-- errors. We effectively ignore this code so that the @action of the
	-- preceding error is used.
	if @error_code = 2601 or @error_code = 547 or @error_code = 3621
		or @error_code = 2627
	    -- ignore the error and keep going
	    select @action = 1000
	else
	    -- abort the synchronization
	    select @action = 3000
	
    END

go


    create function ml_qa_get_agent_prop(
	@agent	varchar(128),
	@pname	varchar(128) )
	returns	varchar(8000)
    as
    begin
	declare @pvalue		varchar(8000)
	declare @row_exists	integer
	select @row_exists = 0
	select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
	    from ml_qa_global_props
	    where client = @agent and name = @pname
	if @row_exists = 0
	    select @pvalue = cast( value as varchar(8000) )
		from ml_qa_global_props
		where client = 'ianywhere.server.defaultClient' and name = @pname
	return @pvalue
    end
go

    create function ml_qa_get_agent_object_prop(
	@agent	varchar(128),
	@objname	varchar(128),
	@pname	varchar(128) )
	returns	varchar(8000)
    as
    begin
	declare @thisproc	sysname
	declare @pvalue		varchar(8000)
	declare @row_exists	integer
	-- This gets around spurious warning about a self-referencing function
	select @thisproc = object_name( @@procid )
	select @row_exists = 0
	select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
	    from ml_qa_global_props
	    where client = @agent and name = @objname + '.' + @pname
	if @row_exists = 0 begin
	    select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
		from ml_qa_global_props
		where client = '' and name = @objname + '.' + @pname
	    if @row_exists = 0 begin
		select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
		    from ml_qa_global_props
		    where client = @agent and name = @objname + '.type'
		if @row_exists = 1
		    if @pvalue is not null
			execute @pvalue = @thisproc @agent, @pvalue, @pname
		else begin
		    select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
			from ml_qa_global_props
			where client = '' and name = @objname + '.type'
		    if @pvalue is not null
			execute @pvalue = @thisproc @agent, @pvalue, @pname
		end
	    end
	end
	return @pvalue
    end
go

    create function ml_qa_get_agent_network_prop(
	@agent	varchar(128),
	@pname	varchar(128) )
	returns	varchar(8000)
    as
    begin
	declare @pvalue	varchar(8000)
	declare @netval	varchar(8000)
	select @netval = 'MAC:' + cast( value as varchar(8000))
	    from ml_qa_global_props
	    where client = @agent and name =  'ias_Network.MAC'
	if @netval is not null
	    execute @pvalue = ml_qa_get_agent_object_prop @agent, @netval, @pname
	if @pvalue is null begin
	    select @netval = 'IP:' + cast( value as varchar(8000))
		from ml_qa_global_props
		where client = @agent and name =  'ias_Network.IP'
	    if @netval is not null
		execute @pvalue = ml_qa_get_agent_object_prop @agent, @netval,
		    @pname
	    if @pvalue is null begin
		select @netval = 'RAS:' + cast( value as varchar(8000))
		    from ml_qa_global_props
		    where client = @agent and name =  'ias_Network.RAS'
		if @netval is not null
		    execute @pvalue = ml_qa_get_agent_object_prop @agent, @netval,
			@pname
		if @pvalue is null begin
		    select @netval = cast( value as varchar(8000))
			from ml_qa_global_props
			where client = @agent and name =  'ias_Network.Adapter'
		    if @netval is not null
			execute @pvalue = ml_qa_get_agent_object_prop @agent,
			    @netval, @pname
		end
	    end
	end
	return @pvalue
    end
go

    create function ml_qa_get_message_prop(
	@agent	varchar(128),
	@pmsgid	varchar(128),
	@pname	varchar(128) )
	returns	varchar(8000)
    as
    begin
	declare @pvalue		varchar(8000)
	declare @row_exists	integer
	select @row_exists = 0
	select @row_exists = 1, @pvalue = cast( value as varchar(8000) )
	    from ml_qa_repository_props
	    where msgid = @pmsgid and name = @pname
	if @row_exists = 0 and @agent is not null
	    execute @pvalue = ml_qa_get_agent_prop @agent, @pname
	return @pvalue
    end
go



create table ml_qa_repository_staging (
	seqno		numeric(20)		not null,
	msgid		varchar(128)	not null,
	destination	varchar(128)	not null,
	originator	varchar(128)	null,
	status		varchar( 255 )	not null,
	statustime	datetime	not null,
	priority	integer		not null,
	expires		datetime	null,
	kind		integer		not null,
	contentsize	numeric(20)		not null,
	props		image	null,
	content		image	null,
	mluser		varchar(128)	null,
	primary key(msgid) )

go

create table ml_qa_status_staging (
	msgid		varchar(128)	not null,
	status		varchar( 255 )	not null,
	statustime	datetime	not null,
	mluser		varchar(128)	null,
	primary key(msgid) )

go

create index ml_qa_mstage_user on ml_qa_repository_staging (
		mluser )
go

create index ml_qa_sstage_user on ml_qa_status_staging ( mluser )
go



    create procedure ml_qa_stage_status_from_client
	@currUser	varchar(128), 
	@tableName	varchar(128)
    as
	declare @msgid varchar(128)
	declare @addr varchar(255)
	declare @client varchar(128)
	declare @status integer
	declare @statustime datetime
	declare scurs cursor for
	    select ss.msgid,
		    case ss.status
			when 'pending' then 1
			when 'expired' then 30
			when 'cancelled' then 40
			when 'unreceivable' then 50
			when 'received' then 60
			else 1 end,
		    ss.statustime, mm.address, mm.originator
		from ml_qa_status_staging ss, ml_qa_messages mm
		where ss.mluser = @currUser and ss.msgid = mm.msgid

	update ml_qa_status_staging set mluser = @currUser where
	    mluser is null

	open scurs
	fetch scurs into @msgid, @status, @statustime, @addr, @client
	while( @@fetch_status = 0 )
	begin
	    exec ml_qa_add_delivery @msgid, @addr, @client, @status,
		@statustime, null
	    fetch scurs into @msgid, @status, @statustime, @addr, @client
	end

	delete ml_qa_status_staging where mluser = @currUser
go

    create procedure ml_qa_staged_status_for_client
	@lastDwnld	datetime,
	@currUser	varchar(128),
	@tableName	varchar(128)
    as
	delete ml_qa_status_staging where mluser = @currUser
go



